# coding: utf-8

"""
    Antimatter Public API

    This API lets you interact with Antimatter Domains and Capsules

    The version of the OpenAPI document: 1.0.0
    Contact: support@antimatter.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictBool, StrictStr, field_validator
from pydantic import Field
from typing_extensions import Annotated
from antimatter.client.models.read_context_config_rule import ReadContextConfigRule
from antimatter.client.models.read_context_parameter import ReadContextParameter
from antimatter.client.models.read_context_required_hook import ReadContextRequiredHook
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class ReadContextDetails(BaseModel):
    """
    Details about a read context 
    """ # noqa: E501
    name: Annotated[str, Field(strict=True)] = Field(description="A human readable name for a read context")
    summary: Annotated[str, Field(strict=True, max_length=140)] = Field(description="A short, single sentence description of this read context")
    description: Annotated[str, Field(strict=True, max_length=4096)] = Field(description="A longer form description of this read context")
    required_hooks: Optional[List[ReadContextRequiredHook]] = Field(default=None, alias="requiredHooks")
    read_parameters: List[ReadContextParameter] = Field(alias="readParameters")
    rules: List[ReadContextConfigRule]
    imported: StrictBool = Field(description="true if this read context is imported")
    source_domain_id: Optional[Annotated[str, Field(strict=True)]] = Field(default=None, description="A globally unique identifier for a domain", alias="sourceDomainID")
    source_domain_name: Optional[StrictStr] = Field(default=None, alias="sourceDomainName")
    __properties: ClassVar[List[str]] = ["name", "summary", "description", "requiredHooks", "readParameters", "rules", "imported", "sourceDomainID", "sourceDomainName"]

    @field_validator('name')
    def name_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^((dm-[1-9A-HJ-NP-Za-km-z]{11}|[a-z][a-z0-9_]{2,31})::)?[a-z][a-z0-9_]{2,31}$", value):
            raise ValueError(r"must validate the regular expression /^((dm-[1-9A-HJ-NP-Za-km-z]{11}|[a-z][a-z0-9_]{2,31})::)?[a-z][a-z0-9_]{2,31}$/")
        return value

    @field_validator('source_domain_id')
    def source_domain_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^dm-[1-9A-HJ-NP-Za-km-z]{11}$", value):
            raise ValueError(r"must validate the regular expression /^dm-[1-9A-HJ-NP-Za-km-z]{11}$/")
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ReadContextDetails from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in required_hooks (list)
        _items = []
        if self.required_hooks:
            for _item in self.required_hooks:
                if _item:
                    _items.append(_item.to_dict())
            _dict['requiredHooks'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in read_parameters (list)
        _items = []
        if self.read_parameters:
            for _item in self.read_parameters:
                if _item:
                    _items.append(_item.to_dict())
            _dict['readParameters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in rules (list)
        _items = []
        if self.rules:
            for _item in self.rules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['rules'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ReadContextDetails from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "summary": obj.get("summary"),
            "description": obj.get("description"),
            "requiredHooks": [ReadContextRequiredHook.from_dict(_item) for _item in obj.get("requiredHooks")] if obj.get("requiredHooks") is not None else None,
            "readParameters": [ReadContextParameter.from_dict(_item) for _item in obj.get("readParameters")] if obj.get("readParameters") is not None else None,
            "rules": [ReadContextConfigRule.from_dict(_item) for _item in obj.get("rules")] if obj.get("rules") is not None else None,
            "imported": obj.get("imported"),
            "sourceDomainID": obj.get("sourceDomainID"),
            "sourceDomainName": obj.get("sourceDomainName")
        })
        return _obj


