# coding: utf-8

"""
    Antimatter Public API

    This API lets you interact with Antimatter Domains and Capsules

    The version of the OpenAPI document: 1.0.0
    Contact: support@antimatter.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List
from pydantic import BaseModel, StrictStr, field_validator
from pydantic import Field
from typing_extensions import Annotated
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class WriteContextConfigInfoRequiredHooksInner(BaseModel):
    """
    WriteContextConfigInfoRequiredHooksInner
    """ # noqa: E501
    hook: Annotated[str, Field(strict=True)] = Field(description="An identifier for a hook")
    constraint: Annotated[str, Field(strict=True)] = Field(description="A pattern for expressing hook version requirements. Note that this is used for expressing two things. The first is which major version of the hook to run on new data. Here, if the version starts with > or no special character, the newest version of the hook is used. If it starts with ~, then the major version is respected, but the newest minor and patch version available on the server will be used. The second function of this pattern is to express a minimum version for read contexts such that if the capsule does not have hook data meeting the requirement, the hook data will be generated just in time. For this, you may express ~1.2.3 to say \"if the hook data is older than 1.2.3, generate it with 1.x.x\" or you may say >1.2.3 to say \"if the hook data is older than 1.2.3, generate it with the newest version\" (which could be 2.x.x). ")
    mode: StrictStr = Field(description="how should this hook be invoked")
    __properties: ClassVar[List[str]] = ["hook", "constraint", "mode"]

    @field_validator('hook')
    def hook_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z][a-z0-9-]{2,32}$", value):
            raise ValueError(r"must validate the regular expression /^[a-z][a-z0-9-]{2,32}$/")
        return value

    @field_validator('constraint')
    def constraint_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^(?:\^|>)?\d+\.\d+\.\d+$", value):
            raise ValueError(r"must validate the regular expression /^(?:\^|>)?\d+\.\d+\.\d+$/")
        return value

    @field_validator('mode')
    def mode_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('sync', 'async'):
            raise ValueError("must be one of enum values ('sync', 'async')")
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of WriteContextConfigInfoRequiredHooksInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of WriteContextConfigInfoRequiredHooksInner from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "hook": obj.get("hook"),
            "constraint": obj.get("constraint"),
            "mode": obj.get("mode") if obj.get("mode") is not None else 'sync'
        })
        return _obj


