import pytest

from antimatter.datatype.datatypes import Datatype
from antimatter.errors.errors import HandlerFactoryError
from antimatter.handlers import factory

datatypes = [dt for dt in Datatype if dt is not Datatype.Unknown]


class TestHandlerFactory:
    """
    TestHandlerFactory is a safeguard test to ensure that every datatype has an
    associated handler generated by the handler factory.
    """

    @pytest.mark.parametrize("datatype", datatypes, ids=[str(dt) for dt in datatypes])
    def test_datatype(self, datatype: Datatype):
        handler = factory(datatype)
        assert handler is not None

    def test_unknown_datatype(self):
        with pytest.raises(HandlerFactoryError):
            factory(Datatype.Unknown)
